package cn.com.ty.lift.business.contract.service;

import cn.com.ty.lift.business.contract.dao.entity.Contracts;
import cn.com.ty.lift.business.contract.dao.entity.ContractsHistory;
import cn.com.ty.lift.business.contract.dao.entity.model.ContractRequest;
import cn.com.ty.lift.business.contract.dao.entity.model.ContractsHistoryRequest;
import cn.com.ty.lift.business.contract.dao.mapper.ContractsHistoryMapper;
import cn.com.ty.lift.business.contract.dao.mapper.ContractsMapper;
import cn.com.ty.lift.business.framework.BusinessBasicException;
import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.common.utils.DateUtils;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * @author bieao
 * @date 2019/12/07
 * @description 合同管理业务层
 */
@Service
public class ContractService {

    @Resource
    private ContractsMapper contractsMapper;

    @Resource
    private ContractsHistoryMapper contractsHistoryMapper;

    @Resource
    private PaymentService paymentService;

    /**
     * @param request 合同列表查询条件
     * @return RestResponse 合同分页列表结果
     * @description 查询合同列表
     * @date 2019/12/7 11:53 AM
     */
    public IPage<Contracts> list(ContractRequest request) {
        IPage<Contracts> page = new Page<>(request.getPageNum(), request.getPageSize());
        return contractsMapper.findByCondition(page, request);
    }

    /**
     * @param request 合同列表查询条件
     * @return RestResponse 合同分页列表结果
     * @description 查询合同列表
     * @date 2019/12/7 11:53 AM
     */
    public IPage<ContractsHistory> list(ContractsHistoryRequest request) {
        IPage<ContractsHistory> page = new Page<>(request.getPageNum(), request.getPageSize());
        return contractsHistoryMapper.findByCondition(page, request);
    }

    /**
     * @param contracts 新增合同数据项
     * @return 1.成功, 0.失败, 消息描述
     * @description 新增合同
     * @date 2019/12/7 3:31 PM
     */
    @Transactional(rollbackFor = Exception.class)
    public RestResponse add(Contracts contracts) {
        if (contracts.getNextId() == null) {
            //按当前日期时间戳自动生成合同编号
            String contractId = DateUtils.generateCode();
            contracts.setNextId(Long.parseLong(contractId));
        }
        int result = contractsMapper.insert(contracts);
        if (result > 0) {
            //批量插入收款信息
            boolean ret = paymentService.insertBatch(contracts);
            if (ret) {
                return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.add.success"));
            } else {
                throw new BusinessBasicException("批量插入收款信息失败");
            }
        }
        return RestResponse.fail(MessageUtils.get("msg.add.fail"));
    }

    /**
     * @param contracts 修改合同数据项
     * @return 1.成功, 0.失败, 消息描述
     * @description 合同变更
     * @date 2019/12/7 3:31 PM
     */
    public RestResponse modify(Contracts contracts) {
        int result = contractsMapper.updateById(contracts);
        if (result > 0) {
            return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.modify.success"));
        }
        return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
    }

    /**
     * @param id 主键id
     * @return RestResponse 合同详情
     * @description 查看合同详情
     * @date 2019/12/7 11:50 AM
     */
    public RestResponse detail(Long id) {
        Contracts contracts = contractsMapper.selectById(id);
        if (ObjectUtil.isEmpty(contracts)) {
            return RestResponse.success();
        }
        return RestResponse.success(contracts, MessageUtils.get("msg.query.detail.success"));
    }
}
