package cn.com.ty.lift.system.settings.service.impl;

import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.common.utils.PojoUtils;
import cn.com.ty.lift.common.utils.ProjectUtils;
import cn.com.ty.lift.system.settings.dao.entity.MaintenanceCompany;
import cn.com.ty.lift.system.settings.dao.entity.MtCompanyAttestation;
import cn.com.ty.lift.system.settings.dao.entity.model.CompanyAttestationRequest;
import cn.com.ty.lift.system.settings.dao.entity.model.CompanyAttestationResponse;
import cn.com.ty.lift.system.settings.dao.mapper.MtCompanyAttestationMapper;
import cn.com.ty.lift.system.settings.service.GlobalSetService;
import cn.com.ty.lift.system.settings.service.IMaintenanceCompanyService;
import cn.com.ty.lift.system.settings.service.IMtCompanyAttestationService;
import cn.com.ty.lift.system.user.dao.entity.UserAccount;
import cn.com.ty.lift.system.user.dao.entity.UserInfo;
import cn.com.ty.lift.system.user.dao.entity.model.MenuRequest;
import cn.com.ty.lift.system.user.service.IMenuService;
import cn.com.ty.lift.system.user.service.IRoleService;
import cn.com.ty.lift.system.user.service.IUserAccountService;
import cn.com.ty.lift.system.user.service.IUserInfoService;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 公司认证信息 服务实现类
 * </p>
 *
 * @author huangyuan
 * @since 2019-12-12
 */
@Service
public class MtCompanyAttestationServiceImpl extends ServiceImpl<MtCompanyAttestationMapper, MtCompanyAttestation> implements IMtCompanyAttestationService {

    @Autowired
    private IMaintenanceCompanyService maintenanceCompanyService;

    @Autowired
    private IRoleService roleService;

    @Autowired
    private IMenuService menuService;

    @Autowired
    private GlobalSetService globalSetService;

    @Autowired
    private IUserAccountService userAccountService;

    @Autowired
    private IUserInfoService userInfoService;

    @Override
    @Transactional
    public RestResponse applyAudit(CompanyAttestationRequest companyAttestationRequest) {
        UserInfo userInfo = userInfoService.getByUserId(companyAttestationRequest.getUserId());
        MtCompanyAttestation mtCompanyAttestation = new MtCompanyAttestation(companyAttestationRequest);
        if (mtCompanyAttestation.getId() != null) {
            //如果用户是重新认证，获取认证对象
            MtCompanyAttestation mtCompanyAttestationInfo = this.getById(mtCompanyAttestation.getId());
            //属性赋值，将前端修改的值赋值给原来的认证对象
            PojoUtils.pojoCopy(mtCompanyAttestationInfo, mtCompanyAttestation);
            mtCompanyAttestation = mtCompanyAttestationInfo;
        } else {
            if(userInfo != null){
                //如果是新建,设置创建者名称
                mtCompanyAttestation.setCreateUser(userInfo.getName());
            }
        }

        //获取公司信息，将公司信息设置为审核中状态
        MaintenanceCompany maintenanceCompany = maintenanceCompanyService.getById(companyAttestationRequest.getMtCompanyId());
        maintenanceCompany.setIsCertificated(ApiConstants.CompanyConstants.MAINTENANCE_WAIT_CERTIFICATE);
        //设置公司审核信息为审核中
        mtCompanyAttestation.setIsCertificated(ApiConstants.CompanyConstants.MAINTENANCE_WAIT_CERTIFICATE);
        maintenanceCompanyService.updateById(maintenanceCompany);
        this.saveOrUpdate(mtCompanyAttestation);
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, "发起认证成功");
    }

    @Override
    @Transactional
    public RestResponse auditPass(CompanyAttestationRequest companyAttestationRequest) {
        MtCompanyAttestation mtCompanyAttestation = this.getById(companyAttestationRequest.getId());
        //获取申请的公司id
        Long companyId = mtCompanyAttestation.getMtCompanyId();

        //认证处理
        MaintenanceCompany maintenanceCompany = auditHandle(mtCompanyAttestation, companyAttestationRequest, ApiConstants.CompanyConstants.MAINTENANCE_PASS);
        //1.将认证通过的信息赋值到本来公司信息中
        maintenanceCompany.copyAttestation(mtCompanyAttestation);
        //2.设置企业电梯台量
        maintenanceCompany.setLimitedNum(companyAttestationRequest.getLimitedNum());
        //3.设置用户是否是vip用户
        maintenanceCompany.setVipFlag(companyAttestationRequest.getVipFlag());
        this.updateById(mtCompanyAttestation);
        //4.给企业分配默认角色，并绑定角色默认菜单
        roleService.saveDefaultMessage(companyId);
        //5.保存给企业分配的菜单
        menuService.companyMenu(new MenuRequest(companyId, companyAttestationRequest.getMenuIds()));
        //6.生成企业设置数据
        globalSetService.teamInitSetting(companyId);
        maintenanceCompanyService.updateById(maintenanceCompany);
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, "操作成功");
    }

    @Override
    @Transactional
    public RestResponse auditFail(CompanyAttestationRequest companyAttestationRequest) {
        MtCompanyAttestation mtCompanyAttestation = this.getById(companyAttestationRequest.getId());
        //认证处理
        MaintenanceCompany maintenanceCompany = auditHandle(mtCompanyAttestation, companyAttestationRequest, ApiConstants.CompanyConstants.MAINTENANCE_PASS);
        this.updateById(mtCompanyAttestation);
        maintenanceCompanyService.updateById(maintenanceCompany);
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, "操作成功");
    }

    @Override
    public RestResponse applyList(CompanyAttestationRequest companyAttestationRequest) {
        Long currentUserId = companyAttestationRequest.getCurrentUserId();
        if ((ApiConstants.CompanyConstants.MAINTENANCE_NOT_CERTIFICATE == companyAttestationRequest.getIsCertificated())) {
            //所有管理员都可以查看待审核的信息
            currentUserId = null;
        } else {
            //如果不是待审核的信息，查看用户类型
            if (currentUserId != null) {
                UserAccount currentUser = userAccountService.getByUserId(currentUserId);
                //判断当前用户类型，如用户是管理员或者平台管理员就查看所有申请信息
                if (ApiConstants.UserConstants.TYPE_PLATFORM_ADMIN == currentUser.getType()
                        || ApiConstants.UserConstants.TYPE_SUPER_ADMIN == currentUser.getType()) {
                    currentUserId = null;
                }
            }
        }
        IPage mtCompanyAttestationPage = new Page<>(companyAttestationRequest.getPageNum(), companyAttestationRequest.getPageSize());
        mtCompanyAttestationPage = this.page(mtCompanyAttestationPage, new QueryWrapper<MtCompanyAttestation>()
                .eq(currentUserId != null, "customer_manager_id", currentUserId)
                //通过审核状态筛选申请信息
                .eq("is_certificated", companyAttestationRequest.getIsCertificated())
                .like(StringUtils.isNotBlank(companyAttestationRequest.getQueryCondition()), "contacts_name",
                        companyAttestationRequest.getQueryCondition())
                .or()
                .like(StringUtils.isNotBlank(companyAttestationRequest.getQueryCondition()), "contacts_tel",
                        companyAttestationRequest.getQueryCondition())
        );
        List<CompanyAttestationResponse> companyAttestationResponseList = new ArrayList<>();
        List<MtCompanyAttestation> mtCompanyAttestationList = mtCompanyAttestationPage.getRecords();
        if(mtCompanyAttestationList != null && mtCompanyAttestationList.size() > 0){
            List<Long> customerManagerIdList = ProjectUtils.getAttrList(mtCompanyAttestationList, "customerManagerId", null);
            List<UserInfo> userInfoList = (List<UserInfo>) userInfoService.listByIds(customerManagerIdList);
            Map<Long, UserInfo> customerManagerIdToUserInfo = ProjectUtils.attrToObjMap(userInfoList, "userId", null);
            for(MtCompanyAttestation mtCompanyAttestation : mtCompanyAttestationList){
                CompanyAttestationResponse companyAttestationResponse = new CompanyAttestationResponse(mtCompanyAttestation);
                //设置客户经理名称
                UserInfo userInfo = customerManagerIdToUserInfo.get(mtCompanyAttestation.getCustomerManagerId());
                if(userInfo != null){
                    companyAttestationResponse.setCustomerManagerName(userInfo.getName());
                }
                companyAttestationResponseList.add(companyAttestationResponse);
            }
            mtCompanyAttestationPage.setRecords(companyAttestationResponseList);
        }
        return RestResponse.success(mtCompanyAttestationPage, ApiConstants.RESULT_SUCCESS, "获取公司申请信息成功");
    }

    /***
     * @description
     * @date 2019-12-13 09:32
     * @param mtCompanyAttestation 公司认证对象
     * @param companyAttestationRequest 公司认证请求对象
     * @param auditResult 审核结果
     * @return
     */
    private MaintenanceCompany auditHandle(MtCompanyAttestation mtCompanyAttestation, CompanyAttestationRequest companyAttestationRequest, int auditResult) {
        MaintenanceCompany maintenanceCompany = maintenanceCompanyService.getById(mtCompanyAttestation.getMtCompanyId());
        //设置认证时间
        mtCompanyAttestation.setDealTime(LocalDateTime.now());
        //设置认证理由
        mtCompanyAttestation.setCheckFeedBack(companyAttestationRequest.getCheckFeedBack());
        //设置客服经理
        mtCompanyAttestation.setCustomerManagerId(companyAttestationRequest.getCustomerManagerId());
        //设置认证结果
        mtCompanyAttestation.setIsCertificated(auditResult);
        //设置认证结果
        maintenanceCompany.setIsCertificated(auditResult);
        return maintenanceCompany;
    }
}
