package cn.com.ty.lift.system.user.controller;

import cn.com.ty.lift.common.aliservice.aliyunsms.AliyunSmsUtil;
import cn.com.ty.lift.common.aliservice.constants.AliConstants;
import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.system.user.dao.entity.UserAccount;
import cn.com.ty.lift.system.user.dao.entity.model.UserRequest;
import cn.com.ty.lift.system.user.service.ILoginService;
import cn.com.ty.lift.system.user.service.IUserAccountService;
import cn.com.ty.lift.system.utils.RandomValidateCodeUtil;
import cn.com.xwy.boot.web.dto.RestResponse;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author huangyuan
 * @date 2019-11-29
 * @description 用户登录
 */
@Controller
@RequestMapping("/login")
public class LoginController {

    @Autowired
    private ILoginService loginService;

    @Autowired
    private IUserAccountService userAccountService;

    /**
     * @description 用户注册
     * @date 2019/11/27 10:03 AM
     * @param userRequest 用户参数对象
     * @return
     */
    @PostMapping("/register")
    @ResponseBody
    public RestResponse register(@RequestBody UserRequest userRequest){
        return loginService.register(userRequest);
    }

    @RequestMapping("/changeTeam")
    @ResponseBody
    public RestResponse changeTeam(UserRequest userRequest){
        return loginService.changeTeam(userRequest);
    }

    /**
     * @description
     * @date 2019/11/27 10:03 AM
     * @param request 请求对象
     * @param userRequest 用户参数对象
     * @return
     */
    @RequestMapping("/login")
    @ResponseBody
    public RestResponse login(HttpServletRequest request, @RequestBody UserRequest userRequest){
        return loginService.login(request, userRequest);
    }

    /**
     * @description 获取图片验证码
     * @date 2019/11/27 10:03 AM
     * @param response http相应类
     * @param request http请求类
     * @return
     */
    @RequestMapping("/getImagCode")
    public void getImageCode(HttpServletResponse response, HttpServletRequest request){
        try {
            response.setContentType("image/jpeg");//设置相应类型,告诉浏览器输出的内容为图片
            response.setHeader("Pragma", "No-cache");//设置响应头信息，告诉浏览器不要缓存此内容
            response.setHeader("Cache-Control", "no-cache");
            response.setDateHeader("Expire", 0);
            RandomValidateCodeUtil randomValidateCode = new RandomValidateCodeUtil();
            randomValidateCode.getRandcode(request, response);//输出验证码图片方法
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @description 图形验证码校验
     * @date 2019/11/27 10:03 AM
     * @param request http请求对象
     * @param inputStr 输入字符串
     * @return
     */
    @RequestMapping("/checkImagCode")
    @ResponseBody
    public RestResponse checkVerify(HttpServletRequest request, String inputStr) {
        String random = (String) request.getSession().getAttribute(RandomValidateCodeUtil.RANDOMCODEKEY);
        if(StringUtils.isBlank(inputStr)){
            return RestResponse.fail(ApiConstants.RESULT_ERROR, "验证码为空");
        }
        if (random != null && random.toLowerCase().equals(inputStr.toLowerCase())) {
            //校验成功
            return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, "验证码校验成功");
        }
        //校验失败
        return RestResponse.fail(ApiConstants.RESULT_ERROR, "验证码校验失败");
    }

    /**
     * @description 验证用户手机号是否注册过
     * @date 2019/11/27 10:03 AM
     * @param mobile 手机号
     * @return
     */
    @RequestMapping("/checkMobile")
    @ResponseBody
    public RestResponse checkMobile(String mobile){
        UserAccount userAccount = userAccountService.getByMobile(mobile);
        if(userAccount != null){
            return RestResponse.fail(ApiConstants.RESULT_ERROR, "手机号已经被注册过了");
        }
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, "验证通过");
    }

    /**
     * @description
     * @date 2019/11/27 10:03 AM
     * @param request
     * @param mobile 手机号
     * @return
     */
    @RequestMapping("/smsCode")
    @ResponseBody
    public RestResponse smsCode(HttpServletRequest request, String mobile){
        if(StringUtils.isNotBlank(mobile)){
            String smsCode = AliyunSmsUtil.sendSmsCode(mobile, AliConstants.SmsConstants.TEST_TEMPLATE_CODE);
            if(StringUtils.isNotBlank(smsCode)){
                request.getSession().setAttribute(AliConstants.SmsConstants.SMS_CODE_NAME, smsCode);
                return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, "发送验证码成功");
            }
            return RestResponse.fail(ApiConstants.RESULT_ERROR, "手机号不存在，发送验证码失败");
        }
        return RestResponse.fail(ApiConstants.RESULT_ERROR, "手机号为空");
    }

    /**
     * @description 校验短信验证码
     * @date 2019/11/27 10:03 AM
     * @param request
     * @param inputSmsCode 输入的短信验证码
     * @return
     */
    @RequestMapping("/checkSmsCode")
    @ResponseBody
    public RestResponse checkSmsCode(HttpServletRequest request, String inputSmsCode){
        String smsCode = (String) request.getSession().getAttribute(AliConstants.SmsConstants.SMS_CODE_NAME);
        if(StringUtils.isNotBlank(inputSmsCode) && inputSmsCode.equals(smsCode)){
            return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, "短信验证码校验成功");
        }
        return RestResponse.fail(ApiConstants.RESULT_ERROR, "短信验证码校验失败");
    }

    /**
     * @description 找回密码
     * @date 2019-12-10
     * @param userRequest
     * @return
     */
    @PostMapping("/resetPassword")
    @ResponseBody
    public RestResponse resetPassword(@RequestBody UserRequest userRequest){
        return loginService.resetPassword(userRequest);
    }

}
